@@ -96,6 +96,7 @@ class MdpEnvironment(Environment, ABC):
         )
 
         agent.reward(next_reward.r)
+
         monitor.report(t=t+1, action_reward=next_reward.r)
 
         return self.state.terminal
@@ -132,6 +133,7 @@ class MdpEnvironment(Environment, ABC):
 
         self.SS = SS
         self.RR = RR
+
         self.terminal_states = [s for s in self.SS if s.terminal]
         self.nonterminal_states = [s for s in self.SS if not s.terminal]
         self.state: Optional[MdpState] = None
@@ -717,7 +719,7 @@ class PrioritizedSweepingMdpPlanningEnvironment(MdpPlanningEnvironment):
         :param t: Time step.
         :param a: Action.
         :param agent: Agent.
-        :return: 3-tuple of (1) current state, current action, and next-state, and (2) reward.
+        :return: 2-tuple of (1) 3-tuple of current state, current action, and next-state, and (2) reward.
         """
 
         planning_state, a = self.get_state_action_with_highest_priority()
@@ -884,7 +886,7 @@ class TrajectorySamplingMdpPlanningEnvironment(MdpPlanningEnvironment):
         :param t: Time step.
         :param a: Action.
         :param agent: Agent.
-        :return: 3-tuple of (1) current state, current action, and next-state, and (2) reward.
+        :return: 2-tuple of (1) 3-tuple of current state, current action, and next-state, and (2) reward.
         """
 
         # sample a random action if the given one is not defined by the model
